#ifndef _COMMON_H_
#define _COMMON_H_
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <list>
#include <stdlib.h>
#include <algorithm>
#include <assert.h>
#include <limits>
using namespace std;
enum {SHADOW = 0,OBJ = 1,BG = 2,STRONG_SHADOW = 3, TREE_LEAF = 4, STATIC = 5};
enum EdgeType {LEFT_EDGE, RIGHT_EDGE};
enum TuningStatus { FAILED_NOISY, FAILED_CLEAN, FINISHED, WORKING_PERFORMANCE, WORKING_PARAM };
enum {UPDATE,NOT_UPDATE,INTEGRATE,QUICK_UPDATE,RESET};
const float MAX_VALUE = 250.0f;

extern int wStep,hStep;

const bool isDebugMode = true;

template <class T> class TOMatrix
{
    public:
	typedef typename std::vector<T>::iterator iterator;
	typedef typename std::vector<T>::const_iterator const_iterator;
    public:
	TOMatrix(){
	    m_width = m_height = 0;
	};
	//TOMatrix( int width, int height, const T &v) : m_data( width * height, v), m_width( width ), m_height( height ) {};
	TOMatrix( int width, int height, const T v) : m_data( width * height, v), m_width( width ), m_height( height ) {};
	void init(int width,int height,const T &v){
	    int size = width * height;
	    //m_data.clear();
	    m_data.resize(size);
	    fill(m_data.begin(), m_data.end(), v);
	    m_width = width;
	    m_height = height;
	};
	void set(int col,int line,const T &v){
	    //assert ( isValidIndex (col, line) );
	    m_data[line*m_width + col] = v;
	};
	void get(int col,int line, T & v) const {
	    //assert( isValidIndex ( col, line ) );
	    v = m_data.at(line*m_width + col);
	};
	T get(int col, int line ) const {
	    //assert( isValidIndex ( col, line ) );
	    T t = m_data.at(line*m_width + col);
	    return t;
	};
	T & getRef(int col,int line){
	    //assert( isValidIndex (col, line) );
	    return m_data.at(line*m_width + col);
	};
    const T & getRefConst( int col, int line) const {
	    //assert( isValidIndex (col, line) );
	    return m_data.at(line*m_width + col);
    };
	void setAll(const T & t){
	    fill(m_data.begin(),m_data.end(),t);
	};
	void getSize(int &width, int &height) const{
	    width = m_width; height = m_height;
	};
	void setSize( const int width, const int height) {
	    int size = width * height;
	    m_data.resize(size);
	    m_width = width;
	    m_height = height;
	};

	bool isValidIndex ( int col, int line) const {
        return ( col < m_width && line < m_height && col >= 0 && line >= 0 );
	    /*if( col < m_width && line < m_height && col >= 0 && line >= 0 )
		return true;
	    if( isDebugMode == true ) {
		ofstream flog("log.txt", ios::out | ios::app );
		flog<<endl<<"Error: Invalid index: col, line: "<<col<< ' ' << line << " width, height: "<<m_width << ' ' << m_height;
		flog.close();
	    };
	    //assert( col < m_width && line < m_height && col >= 0 && line >= 0 );
	    return false;*/
	};

	void copy (const TOMatrix <T> & matrix) {
	    m_width = matrix.m_width;
	    m_height = matrix.m_height;
	    m_data = matrix.m_data;
	}
	template <typename Rect>
	    void setRegion( Rect &r, T &value) {
		for(int col = r.xTop; col <= r.xBottom; ++col)
		    for( int line = r.yTop; line <= r.yBottom; ++line) //Axe y of opencv is downwar
			this->getRef( col, line) = value; 
	    };

	iterator begin() { return m_data.begin(); };
	iterator end() { return m_data.end(); };
	const_iterator cbegin() const { return m_data.begin(); };
	const_iterator cend() const { return m_data.end(); };

    private:
	vector<T> m_data;
	int m_width;
	int m_height;
};

struct GaussianParams {
	float _mu, _std, _cutoffThreshold;
	GaussianParams( const float mu, const float std, const float cutoffThreshold) : _mu(mu), _std(std), _cutoffThreshold(cutoffThreshold) {};
	GaussianParams( const GaussianParams & obj);
	GaussianParams() {};
	void load( ifstream &f);
	float computeLogLikelihood( const float x ) const;
};

typedef list<string> strList;
typedef strList::iterator strListIt;

template <typename value_type>
struct GenericParam {
    string name;
    value_type value;
};

typedef list < GenericParam <float> > Params;
typedef Params::iterator IntParamsIt;

//typedef list <Param> ParamSet;
//typedef ParamSet::iterator ParamIt;

typedef vector <int> IVect;
//typedef vector <ParamSet> ParamSets;

template <typename ValueType>
struct GenericParamDefinition {
    typedef ValueType value_type;
    typedef list <value_type> Values;
    string name;
    int initializePeriod;
    int evaluationPeriod;
    value_type defaultValue;
    GenericParamDefinition( value_type value) {
	defaultValue = value;
	initializePeriod = 200;
	evaluationPeriod = 200;
    }
};

struct NumericParamDefinition {
    string name;
    float value;
    float maxValue, minValue;
    float unit;
};

typedef list< NumericParamDefinition> NumericParamDefinitions;



struct BoolParamDefintion: public GenericParamDefinition <bool> {
    Values values;
    BoolParamDefintion(): GenericParamDefinition <bool> (true) {
	values.push_back(true);
	values.push_back(false);
    }
};

const int UNDEFINED = -1;

struct SetParamDefinition: public GenericParamDefinition <int> {
    Values values;
    SetParamDefinition( list <int> &setValues ): GenericParamDefinition <int> (UNDEFINED) {
	values = setValues;
    };
    SetParamDefinition( ): GenericParamDefinition <int> (UNDEFINED) {
    };
};

//typedef list <ParamDefinition> ParamDefinitions;

template <typename Stream>
void saveBoolValue( Stream & stream, const char * comment, bool value) {
    int val = 0;
    if( value == true )
        val = 1;
    stream<< comment<< val;
}

    template <class stream> 
void ReadLabel(stream &f, const char *pcomment) {
	string str;
    f>>str;
    if(str != pcomment)
    {
	cout<<endl<<"Error reading parameter: "<<str<<" : "<<pcomment;
	cout<<endl<<"str size: "<<str.size();
    assert(false);
	exit(1);
    };
};
    template <class T, class stream> 
void ReadParam(stream &f,T &t,const char *pcomment){
    ReadLabel( f, pcomment );
    f>>t;
};

template <typename stream>
void loadBoolValue( stream & f, const char *pcomment, bool &val) {
    int intVal;
    ReadParam( f, intVal, pcomment);
    if( intVal == 1)
        val = true;
    else
        val = false;
};

struct Pos
{
    Pos() { col = line = -1;};
    Pos( int col1, int line1): col( col1), line( line1) {
    };
    int col,line;
    bool isEqual( const Pos p) const {
        return ( col == p.col and line == p.line);
    };
    bool operator == ( const Pos & p) const {
        return isEqual( p );
    };
    void print() const{
        cout<<"\n col, line: "<<col<<' '<<line;
    };
    void add( const Pos p) {
        col += p.col;
        line += p.line;
    };
    void divide( int count) {
        col /= count;
        line /= count;
    };
    void save( ofstream & ostream) const{
        ostream<<"\ncol: "<<col<<" line: "<<line;
    };
    void load( ifstream & istream) {
        ReadParam( istream, col, "col:");
        ReadParam( istream, line, "line:");
    };
};


void RGBtoHSV( int r, int g, int b, int &h, int &s, int &v );
void getNormalizedColor(int red,int green, int blue,int &r,int &g);
struct Pix
{
    Pix()
    {
	r = g = b = 0;
    };
    float r,g,b;
};

struct HSG //h and s in hsv, g in RGB
{
    HSG()
    {
	h = s = g = 0;
    };
    int h,s,g;
};

//IntImg: value from 0-25500
class CIntImg
{
    public:
	typedef TOMatrix< Pix > ::iterator iterator;
	CIntImg()
	{};
	void initImg(int width,int height);
	void clearImg(int val=0);
	int getPix(int col,int line,Pix &p) const;
	int setPix(int col,int line,const Pix &p);
	bool isValidIndex(int i,int j) const;
	void getSize(int &width,int &height) const
	{
	    m_pixMatrix.getSize(width,height);
	}
	void copy( CIntImg &source );
	iterator begin() { return m_pixMatrix.begin(); };
	iterator end() { return m_pixMatrix.end(); };
    private:
	TOMatrix<Pix> m_pixMatrix;
};

template <typename stream>
bool isEof( stream &f) {
    f.peek();
    return f.eof();
};




void ReadPixParam ( fstream &f, Pix & pix, const char *comment);
void WritePixParam( fstream &f, Pix & pix, const char *comment);

typedef list<Pos> PosList;
typedef PosList::iterator PLIt;

struct ShadowTextureVerifier {
    bool isTextureSmooth(int i,int j,TOMatrix<int> &m,TOMatrix <Pix> &diffs, CIntImg &curImg,int adjacentDiff);
};

struct BackgroundTextureVerifier {
    bool isTextureSmooth(int i,int j,TOMatrix<int> &m,TOMatrix <Pix> &diffs, CIntImg &curImg,int adjacentDiff);
};

template <typename TextureVerifier>
void verifyTexture(TOMatrix<int> &m,TOMatrix<Pix> &bgs, CIntImg &curImg,PosList &pList,int adjacentDiff, TextureVerifier & textureVerifier) {
    int width,height,col,line;
    list<Pos> objPixelList;
    Pos pos;
    m.getSize(width,height);
    width--; height--;
    PLIt pIt;
    //CompPosList updateList;
    //cout<<endl<<"Before: "<<cpList.size();
    for(pIt = pList.begin(); pIt != pList.end(); pIt++)
    {
	col = (*pIt).col; line = (*pIt).line;
	if((col > 0) && (col < width) && (line > 0) && (line < height))
	{
	    if(textureVerifier.isTextureSmooth(col,line,m, bgs, curImg,adjacentDiff) == false)//not shadow point
	    {
		pos.col = col; pos.line = line;
		objPixelList.push_back(pos);
	    }
	    //else
	    //	updateList.push_back((*cpIt));//shadow point
	}
    };
    //cout<<endl<<"After: "<<objPixelList.size();
    list<Pos>::iterator it;
    for(it = objPixelList.begin();it != objPixelList.end(); it++)
	m.getRef((*it).col,(*it).line) = OBJ;


};


template <int v>
struct Int2Type
{
    enum {value = v};
};

struct Rect
{
    Rect() { xTop = yTop = xBottom = yBottom = -1;};
    int xTop,yTop,xBottom,yBottom;
    int getWidth() {return xBottom - xTop; };
    int getHeight() { return yBottom - yTop; };
    bool isInsideRect( const Pos &pos) const {
        return (xTop <= pos.col and xBottom >= pos.col and yTop <= pos.line and yBottom >= pos.line);
    };
    void print() const {
	cout<<"\n xTop, yTop, xBottom, yBottom: "<<xTop<<' '<<yTop<<' '<<xBottom<<' '<<yBottom;
    }
    bool isValid() const {
        return ( xTop < xBottom and yTop < yBottom);
    }
};

typedef list<Rect> RectList;
typedef RectList::iterator RectIt;

bool isInsideRect ( int col, int line, Rect &rect);

bool hasIntersection( const Rect & rect1, const Rect & rect2, Rect & intersect);

struct TextureElement {
    int lbp[8];
    TextureElement() {
	fill_n(lbp, 8, 0);
    }
};

struct PixWithTexture {
    Pix pix;
    TextureElement textureElement;
};

void loadPixWithTexture (const char * fileName, list <PixWithTexture> & pixWithTextures );
void getTextureElement ( CIntImg &curImg, int col, int line, TextureElement &textureElement);

bool isSameTexture(TextureElement &texture1, TextureElement &texture2);

void opening ( TOMatrix <int> &inputImage, TOMatrix <int> &outputImage, int FgVal, int BgVal);

void rgb2yuv (Pix &rgb, Pix &yuv);

void yuv2rgb (Pix &yuv, Pix &rgb);

bool isDarker ( Pix & diffPix);

bool isEdgePoint (Pos pos, CIntImg &curImg, TOMatrix <int> &detectionResults, EdgeType & edgeType );

void findObjectPointFromEdge( Pos pos, CIntImg &curImg, EdgeType edgeType, TOMatrix <int> &detectionResults );

void convertStrongShadow( PosList &shadowPos, TOMatrix <int> & detectionResults);

int countValue( TOMatrix <int> & detectionResult, Rect & rect, int value);

int chromaticityKimDistance( Pix & x, Pix & v );

int noisePercentage( TOMatrix <int> & detectionResults, Rect & rect);

int noisePercentage( TOMatrix <int> &rawSegResults, TOMatrix <int> &segResults, TOMatrix <int> &updateMask, Rect & rect);

int eliminateSmallFgRegion( int threshold, int label, TOMatrix <int> &noisyImage);

void frameDifference( CIntImg &prevImg, CIntImg &curImg, int threshold, TOMatrix<int> &result );

void extractInnerFgPixels( TOMatrix <int> & detectionResults );

void applyMorphology( TOMatrix <int> &segResult);



bool isSimilar( CIntImg & curRefImg, CIntImg & oldRefImg, int SimilarThreshold, int & similarity);


struct CalibrationParameter {
    float fx, fy, cx,cy;
};

void loadRect( ifstream & f, Rect &rect );

void saveBorder( const vector<Pos> &border, ofstream &f);

inline float erf( const float x) {
	float denominator = 1 + 0.278393 * x + 0.230389 * x * x + 0.000972 * x * x * x + 0.078108 * x * x * x * x;
	denominator = 1.0f - 1.0 / (denominator * denominator * denominator * denominator);
	return denominator;
};

struct PosVal {
    Pos pos;
    int val;
    bool operator == ( const PosVal & posVal ) { return (pos.col == posVal.pos.col and pos.line == posVal.pos.line and val == posVal.val); };
};

void convertBorderVector( const vector <Pos> & borderPoints, const int originalHeight, const int originalWidth, const int width, const int height, TOMatrix < list <PosVal> > &borderMatrix);

void findLongestRange( const std::vector <int> & v, int mergeDist, int checkValue, std::list <int> & longestRangeElements);

bool detectRange( const std::vector <int> & vv, int mergeDist, int checkValue, int & curLowerIdx, int &curUpperIdx);

void duplicate( const std::vector <int> &v, std::vector <int> & vv );


float computeSimpleScore( float meanValue, float firstThreshold,  float secondThreshold,  float maxScore,  float mediumScore,  float minScore, float value );

float computeExponentialScore( float meanValue, float std, float value);

inline void PressEnterToContinue() {
    std::cout << "Press ENTER to continue... " << flush;
    std::cin.ignore( std::numeric_limits <std::streamsize> ::max(), '\n' );
} 

bool isEmptyFile(std::ifstream& pFile);


#endif
