#ifndef COMMON_H_
#define COMMON_H_

#include <stdio.h>
#include <vector>

#include <string>

//opencv2
#include <opencv2/core/core.hpp>
#include <opencv2/core/core_c.h>
#include <opencv2/imgproc/imgproc_c.h>
#include <opencv2/legacy/compat.hpp>
#include <opencv2/highgui/highgui_c.h>

//opencv1
//#include "highgui.h"
//#include "cv.h"
//#include "cvaux.h"

const std::string RED = "\033[0;31m";
const std::string GREEN = "\033[0;32m";
const std::string YELLOW = "\033[0;33m";
const std::string NORMAL = "\033[0m";

inline std::string makeRed(const std::string& text) {
        return( RED+text+NORMAL );
}

inline std::string makeYellow(const std::string& text) {
        return( YELLOW+text+NORMAL );
}

inline std::string makeGreen(const std::string& text) {
        return( GREEN+text+NORMAL );
}

bool fileExist(const char* filename);

/*
 * display image in window
 */
void show_img(const IplImage* img, const char* title = "img", int w = 0, int h = 0, bool destroy = true);
/*
 * maps values to jetColorMap
 */
void jetColorMap(unsigned char *rgb,float value,float min,float max);
/*
 * load images using info file
 */
std::vector<IplImage*> loadImages(const char* info_file, int amount = -1);
/*
 * release vector of images
 */
void cvReleaseImages(std::vector<IplImage*> images);

IplImage* getGrayImg(const IplImage* img);

IplImage* getDiff(const IplImage* img1, const IplImage* img2);

IplImage* getImageROI(IplImage* img, CvRect rect);

IplImage* getRandomImageROI(IplImage* img, CvSize size);

std::string IntToString(int val);
std::string FloatToString(float val, std::ios_base& (*base)(std::ios_base& str) = std::fixed);

/*
 * returns rectangles which correspond to foreground (fg/ [fg + bg] = fg_bg)
 */
std::vector<CvRect*> getRects(std::vector<IplImage*> imgs, float fg_bg);

IplImage* getRectImage(IplImage* img, float fg_bg);
/*
 * returns rectangle which corresponds to foreground (fg/ [fg + bg] = fg_bg)
 */
CvRect* getRect(IplImage* img, float fg_bg);
CvRect getROI(CvRect, float);

/*
 * returns square image containing pixel values coresponding to mask
 */
IplImage* extractImage(IplImage* img, uchar* mask);

/*
 * returns grid of rectangles (4 rectangles)
 */
CvRect* get4Rect(CvRect rect);

/*
 * returns grid of rectangles (16 rectangles)
 */
CvRect* get16Rect(CvRect rect);

/*
 * width and height return number of patterns extracted horizontally and vertically, respectively
 */
std::vector<CvRect> getPatterns(CvRect window, CvSize pattern, CvSize step, int* width = NULL, int* height = NULL);

/*
 * returns histogram intersection
 */
double getINTERSECT(IplImage* img1, IplImage* img2, CvRect* rect1 = NULL, CvRect* rect2 = NULL);

CvHistogram* getHist_1D(IplImage* im1, CvRect* rect1, int size, bool normalize = true);
CvHistogram* getHist_3D(IplImage* im1, CvRect* rect1, int* sizes, bool normalize = true);

CVAPI(void) cvEqualize3Hist( const IplImage* src, IplImage* dst );

CVAPI(void) cvEqualize3HistMask( const IplImage* src, IplImage* dst );
/*
 * features is a string representing binary vector
 *  functions converts string to vector
 */
std::vector<bool> s2v(std::string features);
std::string v2s(std::vector<bool> vector);

std::string addbinary(std::string a, std::string b);

void _MSG(std::string msg);
void _WARNING(std::string msg);
void _ERROR(std::string msg);

#endif /* COMMON_H_ */
