/* supDataActor.h ---
 *
 * Author: %Author%
 */

/* Commentary:
 *
 */

/* Change log:
 *
 */

#ifndef SUPDATAACTOR_H
#define SUPDATAACTOR_H


#include "supCoreExport.h"
#include "supAbstractData.h"
#include "boundingbox3d.h"
#include "information3d.h"

#include "trajectory3D.h"
#include "skeletonStructure.h"
#include "supDataLinkedPhysicalObject.h"

class supDataActorPrivate;

// /////////////////////////////////////////////////////////////////
// supDataActor interface
// /////////////////////////////////////////////////////////////////

class SUPCORE_EXPORT supDataActor : public supAbstractData
{
    Q_OBJECT

    public:
        supDataActor(void);
        supDataActor(const supDataActor& other);
        virtual ~supDataActor(void);

    public:
        virtual supDataActor *clone(void); // If your class is pure virtual you need to set this method to pure virtual too! 

    public:
        supDataActor& operator = (const supDataActor& other);

    protected:
        virtual void copy(const dtkAbstractObject& other);

    public:
        QString identifier(void) const;

    public:
        // Add your own setter methods
        
        void setIdent( const unsigned int id);

        //! Change the type of an actor.
        //! \param i_newType Name of the new type to be assigned to the actor.
        void setType( const std::string newType );


        //! Set the bounding box 3D of the actor if the underlying implementation allows it.
        void setBoundingBox3D( const BoundingBox3D& );
        void setInformation3D( const Information3D& );


        //! Sets the bounding box 2d if the implementation allows it.
        void setBoundingBox2D( const cv::Rect & );

        void setCurrentPosition( const cv::Point3f & position );

        void setTrajectory(const CTrajectory3D &);
        void updateTrajectory( const CSynchronizer & sync );


        void addLinkedPhysicalObject(const SpLinkedPhysicalObject );

        void setLPOList(const SpLinkedPhysicalObjectList &);

        void setSkeleton( const SkeletonStructure & skeleton);

        void setSync(const CSynchronizer &i_sync);

        void releaseTPO();
        
///////////for Baptiste //////////////////////////////
        void setPixels( std::list < cv::Point2i > & pixels);
		void setVoxels( std::list < cv::Point3f > & voxels);
///////////for Baptiste //////////////////////////////

        void setLPO (const SpLinkedPhysicalObject&);

        void setSpeed(double speed); // unit m/s

        void setHasSon(bool hs);

        void setReliability(double reli);

		void setCameraId (unsigned int i_camId);
		void setVtRefIds (std::vector <unsigned int> i_vtRefIds);
    

        // Try to sort in the alphabetical order

    public:
        // Add your own getter methods
        unsigned int getIdent() const;

        //! \return The type of the actor.
        std::string getType() const;

        //! \return The bounding box 3D of the actor.
        BoundingBox3D getBoundingBox3D() const;
        Information3D getInformation3D() const;

        //! \return The bounding box 2D of the actor.
        cv::Rect getBoundingBox2D() const;

        //! \return The trajectory followed by the actor.
        const CTrajectory3D & getTrajectory() const;
        
        cv::Point3f getCurrentPosition() const;

        cv::Point3f getHead3dPosition() const;


        cv::Point2i getHead2dPosition() const;

        SpLinkedPhysicalObject getLPO() const;

        SpLinkedPhysicalObjectList getLinkedPhysicalObjectList() const;
        SpLinkedPhysicalObjectList getRefLPOList();

        double getSpeed() const;

        bool hasSon() const;

///////////for Baptiste //////////////////////////////
        void getPixels( std::list < cv::Point2i > & pixels) const;
		void getVoxels( std::list < cv::Point3f > & voxels) const;
///////////for Baptiste //////////////////////////////

		double getReliability() const;
		CSynchronizer getSync() const;

		unsigned int getCameraId() const;
		std::vector <unsigned int> getVtRefIds () const;
    
    
        //sup::SpClassificationProperties getProperties() const;
        // Try to sort in the alphabetical order

    public:
        // Methods you want to overload from dtkAbstractData

        // Try to sort in the alphabetical order

    public:
        // Miscellaneous methods

        // Try to sort in the alphabetical order

    private:
        DTK_DECLARE_PRIVATE(supDataActor);
};

typedef std::list< supDataActor > Actors;

// dpchau
//typedef supDataActor supDataTrackedPhysicalObject;
//typedef QSharedPointer < supDataTrackedPhysicalObject > SpTrackedPhysicalObject;
//typedef std::list< SpTrackedPhysicalObject > SpTrackedPhysicalObjectList;


Q_DECLARE_METATYPE(supDataActor)  // If your class is pure virtual, you cannot register to QMetatype.
Q_DECLARE_METATYPE(supDataActor*) // Hence, you need to comment theses lines.

#endif

