#ifndef SUPVIEWRGBD_H
#define SUPVIEWRGBD_H

#include "supCoreExport.h"

#include <dtkCore/dtkAbstractView.h>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

#include <supCore/supDataImage.h>
#include <supCore/supDataBlob.h>
#include <supCore/supDataPhysicalObject.h>
#include <supCore/supDataActor.h>
#include <supCore/supDataEvent.h>
#include <supCore/ContextElement.h>
#include <supCore/supAbstractDataCalibrationWithDepth.h>


// Predicate for convex envelope
bool point2i_is_lower (cv::Point2i first_point, cv::Point2i second_point);

class supViewRGBDPrivate;

class SUPCORE_EXPORT supViewRGBD : public dtkAbstractView
{
    Q_OBJECT

public:
             supViewRGBD(void);
             supViewRGBD(const supViewRGBD& other);
    virtual ~supViewRGBD(void);

public:

    void setContextInfo ( std::list <ContextElement> &context );
    void setGroundPlaneCoef ( std::vector<double> newCoef );
    void setCalibrationWithDepth ( supAbstractDataCalibrationWithDepth calib );

    void setAcquisitionImage( QSharedPointer< supDataImage >  i_acquisition_image );
    void setAcquisitionDepth( QSharedPointer< supDataImage > i_acquisition_depth );
    void setActors( std::list< QSharedPointer< const supDataActor> > &actors);
    void setEvents( std::list<QSharedPointer<supDataEvent> > listOfEvents);

    void setSegmentationImage( QSharedPointer< supDataImage >  i_segmentation_image );
    void setBlobs( QList< QSharedPointer< supDataBlob > > i_blobs );
    void setPhysicalObjects( QList< QSharedPointer< supDataPhysicalObject > > i_pos );

    void setActorsFeetPositions (std::map <int, std::pair <cv::Point3f, cv::Point3f> > listOfFeetPositions);
    void setFrameRate (int frameRate);

    void setListOfPointsDEBUG (std::list <cv::Point3f> listOfPointsDEBUG);
    void setListOfGroundPoints (std::list <cv::Point3f> listOfGroundPoints);

public:

    cv::Point2i convertWorld2ImageFixed( const cv::Point3f point3D );

    void updateDisplay(void);
    void initColors ();
    QSharedPointer< supDataImage >& getDisplay(void);

    void drawZones ();
    void drawGroundPlane();
    void drawActors ();
    void drawBlobs ();
    void drawTraj (int trajSize);
    void drawEvents ();
    void drawActorInfos ();
    void drawFeet ();
    void drawFrameRate();
    void drawListOfPointsDEBUG();
    void drawListOfGroundPoints();

private:

    std::vector< cv::Point2i > computeConvexEnvelope ( std::list< cv::Point2i > listOfPoint );

public:
    QWidget* widget(void);

private:
    DTK_DECLARE_PRIVATE(supViewRGBD)
};

// /////////////////////////////////////////////////////////////////
// 
// /////////////////////////////////////////////////////////////////

SUPCORE_EXPORT dtkAbstractView *createsupViewRGBD(void);

#endif
